﻿#include <iostream>
#include <iomanip> // для setw()
#include <cassert> // для assert()
#include <ctime>  // для srand()
#define M 5
#define N 5

using namespace std;

template <class T>
class Container
{
private:
    int m_length;
    T* m_data; // параметризированный массив

public:
    Container() : m_length(0), m_data(nullptr) {} // Кнструктор по умолчанию
    Container(int length) : m_length(length) {  // Конструктор с парметрами
        assert(length >= 0);
        if (length > 0)
            m_data = new T[length];
        else
            m_data = nullptr;
    }
    Container(const std::initializer_list<T>& list) : Container(list.size()) { // Конструктор с инициализацией
        int count = 0;
        for (auto& element : list) {
            m_data[count] = element;
            ++count;
        }
    }
    Container(const Container& drob) : m_length(drob.m_length), m_data(drob.m_data) {}   // Конструктор копирования
    ~Container() { delete[] m_data; } // Деструктор

    // Удаление всех элементов контейнера
    void erase() {
        delete[] m_data; m_data = nullptr; m_length = 0;
    }

    // Функция перестановки двух элементов местами
    void swap(int i, int j)
    {
        T tmp = m_data[i];
        m_data[i] = m_data[j];
        m_data[j] = tmp;
    }

    // Оператор обращения к элементу контейнера через квадратные скобки
    T& operator[](int index) {
        assert(index >= 0 && index < m_length);
        return m_data[index];
    }

    // Перегрузка оператора присваивания
    Container& operator= (const Container& a) {
        erase();
        assert(a.m_length>0);
        m_data = new T[a.m_length];
        m_length = a.m_length;
        return *this;
    }

    // Перегрузка оператора присваивания (с использованием инициализирующего списка)
    Container& operator=(const std::initializer_list<T>& list) {
        // Если новый список имеет другой размер, то перевыделяем его
        if (list.size() != static_cast<size_t>(m_length)) {
            delete[] m_data;
            m_length = list.size();
            m_data = new T[m_length];
        }
        // Теперь инициализируем массив значениями из списка
        int count = 0;
        for (auto& element : list) {
            m_data[count] = element;
            ++count;
        }
        return *this;
    }

    int getLength() { return m_length; }

    // Объявление перегруженного оператора вывода другом класса, чтобы иметь доступ к его закрытым членам
    friend ostream& operator <<<T>(ostream& os, const Container& rhs);
};


// Перегрузка оператора вывода
template <typename T>
std::ostream& operator<<(std::ostream& os, const Container<T>& a) {
    // Поскольку перегруженный оператор вывода - друг класса Container, мы можем обращаться к m_length и m_data
    for (int i = 0; i < a.m_length; i++)
    {
        cout << a.m_data[i] << endl;
    }
    return os;
}

int main()
{
    Container<int> c(M * N); // Создание контейнера типа int размером M x N
    int i, j;

    // Создаем матрицу
    srand(time(0));
    for (i = 0; i < M; i++) {
        for (j = 0; j < N; j++) {
            c[i*M+j] = rand() % 101 - 50;
            cout << setw(4) << c[i * M + j];
        }
        cout << endl;
    }

    // Располагаем положительные элементы в начале (сортировка по убыванию)
    for (i = 0; i < c.getLength() - 1; ++i)
    {
        int k = i;
        for (j = i + 1; j < c.getLength(); ++j)
            if (c[j] > c[k]) k = j;
        c.swap(i, k);
    }

    cout << endl << endl << endl;
    // Результат
    for (i = 0; i < M; i++) {
        for (j = 0; j < N; j++) cout << setw(4) << c[i * M + j];
        cout << endl;
    }

    return 0;
}